@@ -30,6 +30,25 @@ const Route = imports.route;
 const RouteQuery = imports.routeQuery;
 const Utils = imports.utils;
 
+/**
+ * Directional sign from the GraphHopper API.
+ * https://github.com/graphhopper/graphhopper/blob/master/docs/web/api-doc.md
+ */
+var Sign = {
+    KEEP_LEFT: -7,
+    TURN_SHARP_LEFT: -3,
+    TURN_LEFT: -2,
+    TURN_SLIGHT_LEFT: -1,
+    CONTINUE_ON_STREET: 0,
+    TURN_SLIGHT_RIGHT: 1,
+    TURN_RIGHT: 2,
+    TURN_SHARP_RIGHT: 3,
+    FINISH: 4,
+    REACHED_VIA: 5,
+    USE_ROUNDABOUT: 6,
+    KEEP_RIGHT: 7
+}
+
 var GraphHopper = class GraphHopper {
 
     get route() {
@@ -85,7 +104,7 @@ var GraphHopper = class GraphHopper {
         this._queryGraphHopper(points, transportationType,
                                (result, exception) => {
             if (exception) {
-                Utils.debug(e);
+                Utils.debug(exception);
                 if (this._query.latest)
                     this._query.latest.place = null;
                 else
@@ -189,7 +208,7 @@ var GraphHopper = class GraphHopper {
             time:        0,
             turnAngle:   0
         });
-        let rest = instructions.map((instr) => {
+        let rest = this._foldInstructions(instructions).map((instr) => {
             let type = this._createTurnPointType(instr.sign);
             let text = instr.text;
             if (type === Route.TurnPointType.VIA) {
@@ -209,6 +228,55 @@ var GraphHopper = class GraphHopper {
         return [startPoint].concat(rest);
     }
 
+    _splitStreetNames(names) {
+        return names.split(',').map(n => n.trim());
+    }
+
+    /**
+     * Returns names found in both the arrays names1 and names2
+     */
+    _namesIntersection(names1, names2) {
+        return names1.filter(Set.prototype.has, new Set(names2));
+    }
+
+    _concatStreetNames(names) {
+        return names.join(', ');
+    }
+
+    _foldInstructions(instructions) {
+        let currInstruction = instructions[0];
+        let res = [];
+
+        for (let i = 1; i < instructions.length; i++) {
+            let newInstruction = instructions[i];
+            let newSign = newInstruction.sign;
+            let currStreetnames = this._splitStreetNames(currInstruction.street_name);
+            let newStreetnames = this._splitStreetNames(newInstruction.street_name);
+            let namesIntersect =
+                this._namesIntersection(currStreetnames, newStreetnames);
+
+            /* if the direction is to continue straight, or keep left or keep
+             * right on the same subset of street/road numbers, fold the instruction into
+             * the previous one
+             */
+            if ((newSign === Sign.CONTINUE_ON_STREET ||
+                 newSign === Sign.KEEP_LEFT || newSign === Sign.KEEP_RIGHT) &&
+                namesIntersect.length > 0) {
+                currInstruction.distance += newInstruction.distance;
+                currInstruction.street_name =
+                    this._concatStreetNames(namesIntersect);
+            } else {
+                res.push(currInstruction);
+                currInstruction = newInstruction;
+            }
+        }
+
+        // push finish instruction
+        res.push(instructions[instructions.length - 1]);
+
+        return res;
+    }
+
     _createTurnPointType(sign) {
         let type = sign + 3;
         let min  = Route.TurnPointType.SHARP_LEFT;
